/*  CRIT_ERR.C
 *  demonstrates special termination when a critical error occurs
 *  last mod.: 17-JAN-91
 */

#include <STDIO.H>
#include <STDLIB.H>
#include <STRING.H>
#include <CONIO.H>

#include "L_MEM.H"

char msg[81];

void main(int argc, char **argv);
Far_str_ptr allocate(Ulong n);

void main(int argc,char **argv)
{
int err_flag;
Far_str_ptr ptr2, ptr3;

if ( argc == 1 || !( argv[1][0] == '0' || argv[1][0] == '1' ) )
    {
    printf("\nUse \"CRIT_ERR 0\" to run this program with the "
        "\nmemory allocation log off.\n"
        "\nUse \"CRIT_ERR 1\" to run this program with the log on;"
        "\nthe log will be dumped to the screen if a critical "
        "\nerror occurs.\n"
        "\nYou will be asked permission to overwrite a control block"
        "\nto demonstrate automatic output of the log when such an"
        "\nerror is detected.\n");
    exit(0);
    }

set_mem_alloc_strategy(FIRST_FIT);

/*  set critical error termination on
 *  regardless of whether the log is on
 */
crit_err_termination_on();

if ( argv[1][0] == '1' )
    {
    mem_log_on();
    /*  specify that report #1 to be output to screen
     *  if a critical error is detected
     */
    set_crit_err_termination_output(SCREEN_OUTPUT,REPORT_1_ONLY);
    }

/*  allocate a few blocks of 10000 bytes  */

allocate(7000L);
ptr2 = allocate(5000L);
ptr3 = allocate(3000L);

printf("\n\nWARNING:  For the purpose of demonstrating program termination"
    "\non detection of control block damage, a control block will now"
    "\nbe overwritten deliberately.  This will probably cause the computer"
    "\nto lock up and a DOS error message to appear.  This is no cause for "
    "\nalarm but you will probably have to reboot the system."
    "\nPress Enter to overwrite the control block, any other to skip.\n");

while ( kbhit() )
    getch();
if ( getch() == '\xD' )
    {
    memset_f(ptr2,0x55,6000L);
    /*  write 6000 bytes, overwriting a control block  */
    printf("\nAttempting to free the third block produces this message:");
    }

/*  attempt to free the third block  */
mem_free(ptr3,&err_flag);

if ( err_flag )
    printf("\n%s",mem_err_msg(err_flag,msg));
    /*  should get a control-blocks_destroyed error message  */
else
    printf("\nBlock at %Fp freed.\n",ptr3);
    /*  should appear only if control block not overwritten  */
}

Far_str_ptr allocate(Ulong n)
{
int err_flag;
Far_str_ptr ptr = mem_alloc(n,&err_flag);

if ( err_flag )
    printf("\n%s",mem_err_msg(err_flag,msg));
else
    printf("\n%lu bytes allocated at %Fp",n,ptr);

return ( ptr );
}
